---
title: http_client
type: input
status: stable
categories: ["Network"]
---

<!--
     THIS FILE IS AUTOGENERATED!

     To make changes please edit the contents of:
     lib/input/http_client.go
-->

import Tabs from '@theme/Tabs';
import TabItem from '@theme/TabItem';


Connects to a server and continuously performs requests for a single message.


<Tabs defaultValue="common" values={[
  { label: 'Common', value: 'common', },
  { label: 'Advanced', value: 'advanced', },
]}>

<TabItem value="common">

```yaml
# Common config fields, showing default values
input:
  label: ""
  http_client:
    url: http://localhost:4195/get
    verb: GET
    headers:
      Content-Type: application/octet-stream
    rate_limit: ""
    timeout: 5s
    payload: ""
    stream:
      enabled: false
      reconnect: true
      codec: lines
```

</TabItem>
<TabItem value="advanced">

```yaml
# All config fields, showing default values
input:
  label: ""
  http_client:
    url: http://localhost:4195/get
    verb: GET
    headers:
      Content-Type: application/octet-stream
    oauth:
      enabled: false
      consumer_key: ""
      consumer_secret: ""
      access_token: ""
      access_token_secret: ""
      request_url: ""
    oauth2:
      enabled: false
      client_key: ""
      client_secret: ""
      token_url: ""
      scopes: []
    jwt:
      enabled: false
      private_key_file: ""
      signing_method: ""
      claims: {}
    basic_auth:
      enabled: false
      username: ""
      password: ""
    tls:
      enabled: false
      skip_cert_verify: false
      enable_renegotiation: false
      root_cas_file: ""
      client_certs: []
    copy_response_headers: false
    rate_limit: ""
    timeout: 5s
    retry_period: 1s
    max_retry_backoff: 300s
    retries: 3
    backoff_on:
      - 429
    drop_on: []
    successful_on: []
    proxy_url: ""
    payload: ""
    drop_empty_bodies: true
    stream:
      enabled: false
      reconnect: true
      codec: lines
      max_buffer: 1000000
```

</TabItem>
</Tabs>

The URL and header values of this type can be dynamically set using function
interpolations described [here](/docs/configuration/interpolation#bloblang-queries).

### Streaming

If you enable streaming then Benthos will consume the body of the response as a continuous stream of data, breaking messages out following a chosen codec. This allows you to consume APIs that provide long lived streamed data feeds (such as Twitter).

### Pagination

This input supports interpolation functions in the `url` and `headers` fields where data from the previous successfully consumed message (if there was one) can be referenced. This can be used in order to support basic levels of pagination. However, in cases where pagination depends on logic it is recommended that you use an [`http` processor](/docs/component/processors/http) instead, often combined with a [`generate` input](/docs/components/inputs/generate) in order to schedule the processor.

## Examples

<Tabs defaultValue="Basic Pagination" values={[
{ label: 'Basic Pagination', value: 'Basic Pagination', },
]}>

<TabItem value="Basic Pagination">

Interpolation functions within the `url` and `headers` fields can be used to reference the previously consumed message, which allows simple pagination.

```yaml
input:
  http_client:
    url: >-
      https://api.example.com/search?query=allmyfoos&start_time=${! (
        (timestamp_unix()-300).format_timestamp("2006-01-02T15:04:05Z","UTC").escape_url_query()
      ) }${! ("&next_token="+this.meta.next_token.not_null()) | "" }
    verb: GET
    rate_limit: foo_searches
    oauth2:
      enabled: true
      token_url: https://api.example.com/oauth2/token
      client_key: "${EXAMPLE_KEY}"
      client_secret: "${EXAMPLE_SECRET}"

rate_limit_resources:
  - label: foo_searches
    local:
      count: 1
      interval: 30s
```

</TabItem>
</Tabs>

## Fields

### `url`

The URL to connect to.
This field supports [interpolation functions](/docs/configuration/interpolation#bloblang-queries).


Type: `string`  
Default: `"http://localhost:4195/get"`  

### `verb`

A verb to connect with


Type: `string`  
Default: `"GET"`  

```yaml
# Examples

verb: POST

verb: GET

verb: DELETE
```

### `headers`

A map of headers to add to the request.
This field supports [interpolation functions](/docs/configuration/interpolation#bloblang-queries).


Type: `object`  
Default: `{"Content-Type":"application/octet-stream"}`  

```yaml
# Examples

headers:
  Content-Type: application/octet-stream
```

### `oauth`

Allows you to specify open authentication via OAuth version 1.


Type: `object`  

### `oauth.enabled`

Whether to use OAuth version 1 in requests.


Type: `bool`  
Default: `false`  

### `oauth.consumer_key`

A value used to identify the client to the service provider.


Type: `string`  
Default: `""`  

### `oauth.consumer_secret`

A secret used to establish ownership of the consumer key.


Type: `string`  
Default: `""`  

### `oauth.access_token`

A value used to gain access to the protected resources on behalf of the user.


Type: `string`  
Default: `""`  

### `oauth.access_token_secret`

A secret provided in order to establish ownership of a given access token.


Type: `string`  
Default: `""`  

### `oauth.request_url`

The URL of the OAuth provider.


Type: `string`  
Default: `""`  

### `oauth2`

Allows you to specify open authentication via OAuth version 2 using the client credentials token flow.


Type: `object`  

### `oauth2.enabled`

Whether to use OAuth version 2 in requests.


Type: `bool`  
Default: `false`  

### `oauth2.client_key`

A value used to identify the client to the token provider.


Type: `string`  
Default: `""`  

### `oauth2.client_secret`

A secret used to establish ownership of the client key.


Type: `string`  
Default: `""`  

### `oauth2.token_url`

The URL of the token provider.


Type: `string`  
Default: `""`  

### `oauth2.scopes`

A list of optional requested permissions.


Type: `array`  
Default: `[]`  
Requires version 3.45.0 or newer  

### `jwt`

BETA: Allows you to specify JWT authentication.


Type: `object`  

### `jwt.enabled`

Whether to use JWT authentication in requests.


Type: `bool`  
Default: `false`  

### `jwt.private_key_file`

A file with the PEM encoded via PKCS1 or PKCS8 as private key.


Type: `string`  
Default: `""`  

### `jwt.signing_method`

A method used to sign the token such as RS256, RS384 or RS512.


Type: `string`  
Default: `""`  

### `jwt.claims`

A value used to identify the claims that issued the JWT.


Type: `object`  
Default: `{}`  

### `basic_auth`

Allows you to specify basic authentication.


Type: `object`  

### `basic_auth.enabled`

Whether to use basic authentication in requests.


Type: `bool`  
Default: `false`  

### `basic_auth.username`

A username to authenticate as.


Type: `string`  
Default: `""`  

### `basic_auth.password`

A password to authenticate with.


Type: `string`  
Default: `""`  

### `tls`

Custom TLS settings can be used to override system defaults.


Type: `object`  

### `tls.enabled`

Whether custom TLS settings are enabled.


Type: `bool`  
Default: `false`  

### `tls.skip_cert_verify`

Whether to skip server side certificate verification.


Type: `bool`  
Default: `false`  

### `tls.enable_renegotiation`

Whether to allow the remote server to repeatedly request renegotiation. Enable this option if you're seeing the error message `local error: tls: no renegotiation`.


Type: `bool`  
Default: `false`  
Requires version 3.45.0 or newer  

### `tls.root_cas_file`

An optional path of a root certificate authority file to use. This is a file, often with a .pem extension, containing a certificate chain from the parent trusted root certificate, to possible intermediate signing certificates, to the host certificate.


Type: `string`  
Default: `""`  

```yaml
# Examples

root_cas_file: ./root_cas.pem
```

### `tls.client_certs`

A list of client certificates to use. For each certificate either the fields `cert` and `key`, or `cert_file` and `key_file` should be specified, but not both.


Type: `array`  

```yaml
# Examples

client_certs:
  - cert: foo
    key: bar

client_certs:
  - cert_file: ./example.pem
    key_file: ./example.key
```

### `tls.client_certs[].cert`

A plain text certificate to use.


Type: `string`  
Default: `""`  

### `tls.client_certs[].key`

A plain text certificate key to use.


Type: `string`  
Default: `""`  

### `tls.client_certs[].cert_file`

The path to a certificate to use.


Type: `string`  
Default: `""`  

### `tls.client_certs[].key_file`

The path of a certificate key to use.


Type: `string`  
Default: `""`  

### `copy_response_headers`

Sets whether to copy the headers from the response to the resulting payload.


Type: `bool`  
Default: `false`  

### `rate_limit`

An optional [rate limit](/docs/components/rate_limits/about) to throttle requests by.


Type: `string`  
Default: `""`  

### `timeout`

A static timeout to apply to requests.


Type: `string`  
Default: `"5s"`  

### `retry_period`

The base period to wait between failed requests.


Type: `string`  
Default: `"1s"`  

### `max_retry_backoff`

The maximum period to wait between failed requests.


Type: `string`  
Default: `"300s"`  

### `retries`

The maximum number of retry attempts to make.


Type: `number`  
Default: `3`  

### `backoff_on`

A list of status codes whereby the request should be considered to have failed and retries should be attempted, but the period between them should be increased gradually.


Type: `array`  
Default: `[429]`  

### `drop_on`

A list of status codes whereby the request should be considered to have failed but retries should not be attempted. This is useful for preventing wasted retries for requests that will never succeed. Note that with these status codes the _request_ is dropped, but _message_ that caused the request will not be dropped.


Type: `array`  
Default: `[]`  

### `successful_on`

A list of status codes whereby the attempt should be considered successful, this is useful for dropping requests that return non-2XX codes indicating that the message has been dealt with, such as a 303 See Other or a 409 Conflict. All 2XX codes are considered successful unless they are present within `backoff_on` or `drop_on`, regardless of this field.


Type: `array`  
Default: `[]`  

### `proxy_url`

An optional HTTP proxy URL.


Type: `string`  
Default: `""`  

### `payload`

An optional payload to deliver for each request.


Type: `string`  
Default: `""`  

### `drop_empty_bodies`

Whether empty payloads received from the target server should be dropped.


Type: `bool`  
Default: `true`  

### `stream`

Allows you to set streaming mode, where requests are kept open and messages are processed line-by-line.


Type: `object`  

### `stream.enabled`

Enables streaming mode.


Type: `bool`  
Default: `false`  

### `stream.reconnect`

Sets whether to re-establish the connection once it is lost.


Type: `bool`  
Default: `true`  

### `stream.codec`

The way in which the bytes of a continuous stream are converted into messages. It's possible to consume lines using a custom delimiter with the `delim:x` codec, where x is the character sequence custom delimiter. It's not necessary to add gzip in the codec when the response headers specify it as it will be decompressed automatically.


Type: `string`  
Default: `"lines"`  
Requires version 3.42.0 or newer  

| Option | Summary |
|---|---|
| `auto` | EXPERIMENTAL: Attempts to derive a codec for each file based on information such as the extension. For example, a .tar.gz file would be consumed with the `gzip/tar` codec. Defaults to all-bytes. |
| `all-bytes` | Consume the entire file as a single binary message. |
| `chunker:x` | Consume the file in chunks of a given number of bytes. |
| `csv` | Consume structured rows as comma separated values, the first row must be a header row. |
| `delim:x` | Consume the file in segments divided by a custom delimiter. |
| `gzip` | Decompress a gzip file, this codec should precede another codec, e.g. `gzip/all-bytes`, `gzip/tar`, `gzip/csv`, etc. |
| `lines` | Consume the file in segments divided by linebreaks. |
| `multipart` | Consumes the output of another codec and batches messages together. A batch ends when an empty message is consumed. For example, the codec `lines/multipart` could be used to consume multipart messages where an empty line indicates the end of each batch. |
| `tar` | Parse the file as a tar archive, and consume each file of the archive as a message. |


```yaml
# Examples

codec: lines

codec: "delim:\t"

codec: delim:foobar

codec: csv
```

### `stream.max_buffer`

Must be larger than the largest line of the stream.


Type: `number`  
Default: `1000000`  


